@@ -116,6 +116,16 @@ IRECV_DEVICES = (
                 display_name='iPhone 15 Pro'),
     IRecvDevice(product_type='iPhone16,2', hardware_model='d84ap', board_id=0x06, chip_id=0x8130,
                 display_name='iPhone 15 Pro Max'),
+    IRecvDevice(product_type='iPhone17,3', hardware_model='d47ap', board_id=0x08, chip_id=0x8140,
+                display_name='iPhone 16'),
+    IRecvDevice(product_type='iPhone17,4', hardware_model='d48ap', board_id=0x0A, chip_id=0x8140,
+                display_name='iPhone 16 Plus'),
+    IRecvDevice(product_type='iPhone17,5', hardware_model='v59ap', board_id=0x04, chip_id=0x8140,
+                display_name='iPhone 16e'),
+    IRecvDevice(product_type='iPhone17,1', hardware_model='d93ap', board_id=0x0C, chip_id=0x8140,
+                display_name='iPhone 16 Pro'),
+    IRecvDevice(product_type='iPhone17,2', hardware_model='d94ap', board_id=0x0E, chip_id=0x8140,
+                display_name='iPhone 16 Pro Max'),
     # iPod
     IRecvDevice(product_type='iPod1,1', hardware_model='n45ap', board_id=0x02, chip_id=0x8900,
                 display_name='iPod Touch (1st gen)'),
@@ -366,6 +376,42 @@ IRECV_DEVICES = (
                 display_name='Apple Watch Series 7 (41mm Cellular)'),
     IRecvDevice(product_type='Watch6,9', hardware_model='n188bap', board_id=0x16, chip_id=0x8301,
                 display_name='Apple Watch Series 7 (45mm Cellular)'),
+    IRecvDevice(product_type='Watch6,10', hardware_model='n143sap', board_id=0x18, chip_id=0x8301,
+                display_name='Apple Watch SE (2nd generation) (40mm)'),
+    IRecvDevice(product_type='Watch6,11', hardware_model='n143bap', board_id=0x2A, chip_id=0x8301,
+                display_name='Apple Watch SE (2nd generation) (44mm)'),
+    IRecvDevice(product_type='Watch6,12', hardware_model='n149sap', board_id=0x2C, chip_id=0x8301,
+                display_name='Apple Watch SE (2nd generation) (40mm Cellular)'),
+    IRecvDevice(product_type='Watch6,13', hardware_model='n149bap', board_id=0x2E, chip_id=0x8301,
+                display_name='Apple Watch SE (2nd generation) (44m Cellular)'),
+    IRecvDevice(product_type='Watch6,14', hardware_model='n197sap', board_id=0x30, chip_id=0x8301,
+                display_name='Apple Watch Series 8 (41mm)'),
+    IRecvDevice(product_type='Watch6,15', hardware_model='n197bap', board_id=0x32, chip_id=0x8301,
+                display_name='Apple Watch Series 8 (45mm)'),
+    IRecvDevice(product_type='Watch6,16', hardware_model='n198sap', board_id=0x34, chip_id=0x8301,
+                display_name='Apple Watch Series 8 (41mm Cellular)'),
+    IRecvDevice(product_type='Watch6,17', hardware_model='n198bap', board_id=0x36, chip_id=0x8301,
+                display_name='Apple Watch Series 8 (45mm Cellular)'),
+    IRecvDevice(product_type='Watch6,17', hardware_model='n199ap', board_id=0x26, chip_id=0x8301,
+                display_name='Apple Watch Ultra'),
+    IRecvDevice(product_type='Watch7,1', hardware_model='n207sap', board_id=0x08, chip_id=0x8310,
+                display_name='Apple Watch Series 9 (41mm)'),
+    IRecvDevice(product_type='Watch7,2', hardware_model='n207bap', board_id=0x0A, chip_id=0x8310,
+                display_name='Apple Watch Series 9 (45mm)'),
+    IRecvDevice(product_type='Watch7,3', hardware_model='n208sap', board_id=0x0C, chip_id=0x8310,
+                display_name='Apple Watch Series 9 (41mm Cellular)'),
+    IRecvDevice(product_type='Watch7,4', hardware_model='n208bap', board_id=0x0E, chip_id=0x8310,
+                display_name='Apple Watch Series 9 (45mm Cellular)'),
+    IRecvDevice(product_type='Watch7,5', hardware_model='n210ap', board_id=0x02, chip_id=0x8310,
+                display_name='Apple Watch Ultra 2'),
+    IRecvDevice(product_type='Watch7,8', hardware_model='n217sap', board_id=0x10, chip_id=0x8310,
+                display_name='Apple Watch Series 10 (42mm)'),
+    IRecvDevice(product_type='Watch7,9', hardware_model='n217bap', board_id=0x12, chip_id=0x8310,
+                display_name='Apple Watch Series 10 (46mm)'),
+    IRecvDevice(product_type='Watch7,10', hardware_model='n218sap', board_id=0x14, chip_id=0x8310,
+                display_name='Apple Watch Series 10 (42mm Cellular)'),
+    IRecvDevice(product_type='Watch7,11', hardware_model='n218bap', board_id=0x16, chip_id=0x8310,
+                display_name='Apple Watch Series 10 (46mm Cellular)'),
     # Apple Silicon Macs
     IRecvDevice(product_type='ADP3,2', hardware_model='j273aap', board_id=0x42, chip_id=0x8027,
                 display_name='Developer Transition Kit (2020)'),
