@@ -3,6 +3,7 @@ import datetime
 import logging
 import os
 import plistlib
+import sys
 import tempfile
 import time
 from abc import ABC, abstractmethod
@@ -10,22 +11,26 @@ from contextlib import contextmanager, suppress
 from enum import Enum
 from functools import wraps
 from pathlib import Path
-from ssl import SSLZeroReturnError
-from typing import Dict, Mapping, Optional
+from ssl import SSLError, SSLZeroReturnError, TLSVersion
+from typing import AsyncIterable, Optional
 
-from cryptography.hazmat.primitives import hashes
+import construct
+from cryptography import x509
+from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey
 from cryptography.hazmat.primitives.serialization import Encoding
-from cryptography.hazmat.primitives.serialization.pkcs7 import PKCS7SignatureBuilder
-from cryptography.hazmat.primitives.serialization.pkcs12 import load_pkcs12
+from cryptography.hazmat.primitives.serialization.pkcs7 import PKCS7Options, PKCS7SignatureBuilder
 from packaging.version import Version
 
 from pymobiledevice3 import usbmux
+from pymobiledevice3.bonjour import DEFAULT_BONJOUR_TIMEOUT, browse_mobdev2
 from pymobiledevice3.ca import ca_do_everything
-from pymobiledevice3.exceptions import CannotStopSessionError, ConnectionTerminatedError, FatalPairingError, \
-    GetProhibitedError, IncorrectModeError, InvalidConnectionError, InvalidHostIDError, InvalidServiceError, \
-    LockdownError, MissingValueError, NotPairedError, PairingDialogResponsePendingError, PairingError, \
-    PasswordRequiredError, SetProhibitedError, StartServiceError, UserDeniedPairingError
+from pymobiledevice3.common import get_home_folder
+from pymobiledevice3.exceptions import BadDevError, CannotStopSessionError, ConnectionFailedError, \
+    ConnectionTerminatedError, DeviceNotFoundError, FatalPairingError, GetProhibitedError, IncorrectModeError, \
+    InvalidConnectionError, InvalidHostIDError, InvalidServiceError, LockdownError, MissingValueError, \
+    NoDeviceConnectedError, NotPairedError, PairingDialogResponsePendingError, PairingError, PasswordRequiredError, \
+    SetProhibitedError, StartServiceError, UserDeniedPairingError
 from pymobiledevice3.irecv_devices import IRECV_DEVICES
 from pymobiledevice3.lockdown_service_provider import LockdownServiceProvider
 from pymobiledevice3.pair_records import create_pairing_records_cache_folder, generate_host_id, \
@@ -34,6 +39,7 @@ from pymobiledevice3.service_connection import ServiceConnection
 from pymobiledevice3.usbmux import PlistMuxConnection
 
 SYSTEM_BUID = '30142955-444094379208051516'
+RESTORED_SERVICE_TYPE = 'com.apple.mobile.restored'
 
 DEFAULT_LABEL = 'pymobiledevice3'
 SERVICE_PORT = 62078
@@ -54,15 +60,21 @@ def _reconnect_on_remote_close(f):
     transmitted). When this happens, we'll attempt to reconnect.
     """
 
+    def _reconnect(self: 'LockdownClient'):
+        self._reestablish_connection()
+        self.validate_pairing()
+
     @wraps(f)
     def _inner_reconnect_on_remote_close(*args, **kwargs):
         try:
             return f(*args, **kwargs)
-        except (BrokenPipeError, ConnectionTerminatedError):
-            self: LockdownClient = args[0]
-
-            self._reestablish_connection()
-            self.validate_pairing()
+        except (BrokenPipeError, ConnectionTerminatedError, SSLError):
+            _reconnect(args[0])
+            return f(*args, **kwargs)
+        except ConnectionAbortedError:
+            if sys.platform != 'win32':
+                raise
+            _reconnect(args[0])
             return f(*args, **kwargs)
 
     return _inner_reconnect_on_remote_close
@@ -70,7 +82,7 @@ def _reconnect_on_remote_close(f):
 
 class LockdownClient(ABC, LockdownServiceProvider):
     def __init__(self, service: ServiceConnection, host_id: str, identifier: str = None,
-                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
+                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Optional[dict] = None,
                  pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT):
         """
         Create a LockdownClient instance
@@ -108,8 +120,9 @@ class LockdownClient(ABC, LockdownServiceProvider):
 
     @classmethod
     def create(cls, service: ServiceConnection, identifier: str = None, system_buid: str = SYSTEM_BUID,
-               label: str = DEFAULT_LABEL, autopair: bool = True, pair_timeout: float = None, local_hostname: str = None,
-               pair_record: Mapping = None, pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT,
+               label: str = DEFAULT_LABEL, autopair: bool = True, pair_timeout: float = None,
+               local_hostname: str = None,
+               pair_record: Optional[dict] = None, pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT,
                private_key: Optional[RSAPrivateKey] = None, **cls_specific_args):
         """
         Create a LockdownClient instance
@@ -142,15 +155,21 @@ class LockdownClient(ABC, LockdownServiceProvider):
         return f'<{self.__class__.__name__} ID:{self.identifier} VERSION:{self.product_version} ' \
                f'TYPE:{self.product_type} PAIRED:{self.paired}>'
 
-    def __enter__(self):
+    def __enter__(self) -> 'LockdownClient':
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
+        self.close()
+
+    async def __aenter__(self) -> 'LockdownClient':
+        return self
+
+    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
         self.close()
 
     @property
     def product_version(self) -> str:
-        return self.all_values.get('ProductVersion')
+        return self.all_values.get('ProductVersion') or '1.0'
 
     @property
     def device_class(self) -> DeviceClass:
@@ -164,8 +183,8 @@ class LockdownClient(ABC, LockdownServiceProvider):
         return self.all_values.get('WiFiAddress')
 
     @property
-    def short_info(self) -> Dict:
-        keys_to_copy = ['DeviceClass', 'DeviceName', 'BuildVersion', 'ProductVersion', 'ProductType']
+    def short_info(self) -> dict:
+        keys_to_copy = ['DeviceClass', 'DeviceName', 'BuildVersion', 'ProductVersion', 'ProductType', 'UniqueDeviceID']
         result = {
             'Identifier': self.identifier,
         }
@@ -228,7 +247,11 @@ class LockdownClient(ABC, LockdownServiceProvider):
         return self.get_value(key='Locale', domain='com.apple.international')
 
     @property
-    def preflight_info(self) -> Mapping:
+    def preflight_info(self) -> dict:
+        return self.get_value(key='PreflightInfo')
+
+    @property
+    def firmware_preflight_info(self) -> dict:
         return self.get_value(key='FirmwarePreflightInfo')
 
     @property
@@ -278,7 +301,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
     def enter_recovery(self):
         return self._request('EnterRecovery')
 
-    def stop_session(self) -> Mapping:
+    def stop_session(self) -> dict:
         if self.session_id and self.service:
             response = self._request('StopSession', {'SessionID': self.session_id})
             self.session_id = None
@@ -310,6 +333,11 @@ class LockdownClient(ABC, LockdownServiceProvider):
 
         self.session_id = start_session.get('SessionID')
         if start_session.get('EnableSessionSSL'):
+            if (Version(self.product_version) < Version('5.0')) and (self.device_class != DeviceClass.WATCH):
+                # TLS v1 is the protocol required for versions prior to iOS 5
+                self.service.min_ssl_proto = TLSVersion.SSLv3
+                self.service.max_ssl_proto = TLSVersion.TLSv1
+
             with self.ssl_file() as f:
                 try:
                     self.service.ssl_start(f)
@@ -335,7 +363,8 @@ class LockdownClient(ABC, LockdownServiceProvider):
             raise PairingError()
 
         self.logger.info('Creating host key & certificate')
-        cert_pem, private_key_pem, device_certificate = ca_do_everything(self.device_public_key, private_key=private_key)
+        cert_pem, private_key_pem, device_certificate = ca_do_everything(self.device_public_key,
+                                                                         private_key=private_key)
 
         pair_record = {'DevicePublicKey': self.device_public_key,
                        'DeviceCertificate': device_certificate,
@@ -362,15 +391,12 @@ class LockdownClient(ABC, LockdownServiceProvider):
         self.paired = True
 
     @_reconnect_on_remote_close
-    def pair_supervised(self, timeout: float = None, p12file: Path = None, password: str = None) -> None:
-
-        keystore_data = p12file.read()
-        try:
-            decrypted_p12 = load_pkcs12(
-                keystore_data, password.encode('utf-8'))
-        except Exception as pkcs12_error:
-            self.service.close()
-            raise Exception(f'load_pkcs12 error: {pkcs12_error}')
+    def pair_supervised(self, keybag_file: Path, timeout: Optional[float] = None) -> None:
+        with open(keybag_file, 'rb') as keybag_file:
+            keybag_file = keybag_file.read()
+        private_key = serialization.load_pem_private_key(keybag_file, password=None)
+        cer = x509.load_pem_x509_certificate(keybag_file)
+        public_key = cer.public_bytes(Encoding.DER)
 
         self.device_public_key = self.get_value('', 'DevicePublicKey')
         if not self.device_public_key:
@@ -379,8 +405,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
             raise PairingError()
 
         self.logger.info('Creating host key & certificate')
-        cert_pem, private_key_pem, device_certificate = ca_do_everything(
-            self.device_public_key)
+        cert_pem, private_key_pem, device_certificate = ca_do_everything(self.device_public_key)
 
         pair_record = {'DevicePublicKey': self.device_public_key,
                        'DeviceCertificate': device_certificate,
@@ -392,17 +417,18 @@ class LockdownClient(ABC, LockdownServiceProvider):
                        'SystemBUID': self.system_buid}
 
         pair_options = {'PairRecord': pair_record, 'ProtocolVersion': '2',
-                        'PairingOptions': {'SupervisorCertificate': decrypted_p12.cert.certificate.public_bytes(Encoding.DER),
-                                           'ExtendedPairingErrors': True}}
+                        'PairingOptions': {
+                            'SupervisorCertificate': public_key,
+                            'ExtendedPairingErrors': True}}
 
-        # first pair with SupervisorCertificate as PairingOptions to get Challenge
+        # first pair with SupervisorCertificate as PairingOptions to get PairingChallenge
         pair = self._request_pair(pair_options, timeout=timeout)
         if pair.get('Error') == 'MCChallengeRequired':
-            extendedresponse = pair.get('ExtendedResponse')
-            if extendedresponse is not None:
-                pairingchallenge = extendedresponse.get('PairingChallenge')
-                signed_response = PKCS7SignatureBuilder().set_data(pairingchallenge).add_signer(
-                    decrypted_p12.cert.certificate, decrypted_p12.key, hashes.SHA256()).sign(Encoding.DER, [])
+            extended_response = pair.get('ExtendedResponse')
+            if extended_response is not None:
+                pairing_challenge = extended_response.get('PairingChallenge')
+                signed_response = PKCS7SignatureBuilder().set_data(pairing_challenge).add_signer(
+                    cer, private_key, hashes.SHA256()).sign(Encoding.DER, [PKCS7Options.Binary])
                 pair_options = {'PairRecord': pair_record, 'ProtocolVersion': '2', 'PairingOptions': {
                     'ChallengeResponse': signed_response, 'ExtendedPairingErrors': True}}
                 # second pair with Response to Challenge
@@ -444,7 +470,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
             return r
 
     @_reconnect_on_remote_close
-    def remove_value(self, domain: str = None, key: str = None) -> Mapping:
+    def remove_value(self, domain: str = None, key: str = None) -> dict:
         options = {}
 
         if domain:
@@ -455,7 +481,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
         return self._request('RemoveValue', options)
 
     @_reconnect_on_remote_close
-    def set_value(self, value, domain: str = None, key: str = None) -> Mapping:
+    def set_value(self, value, domain: str = None, key: str = None) -> dict:
         options = {}
 
         if domain:
@@ -466,7 +492,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
         options['Value'] = value
         return self._request('SetValue', options)
 
-    def get_service_connection_attributes(self, name: str, include_escrow_bag: bool = False) -> Mapping:
+    def get_service_connection_attributes(self, name: str, include_escrow_bag: bool = False) -> dict:
         if not self.paired:
             raise NotPairedError()
 
@@ -539,14 +565,16 @@ class LockdownClient(ABC, LockdownServiceProvider):
         """ Used to establish a new ServiceConnection to a given port """
         pass
 
-    def _request(self, request: str, options: Mapping = None, verify_request: bool = True) -> Mapping:
+    def _request(self, request: str, options: Optional[dict] = None, verify_request: bool = True) -> dict:
         message = {'Label': self.label, 'Request': request}
         if options:
             message.update(options)
         response = self.service.send_recv_plist(message)
 
-        if verify_request and response['Request'] != request:
-            raise LockdownError(f'incorrect response returned. got {response["Request"]} instead of {request}')
+        if verify_request and response.get('Request') != request:
+            if response.get('Type') == RESTORED_SERVICE_TYPE:
+                raise IncorrectModeError(f'Incorrect mode returned. Got: {response}')
+            raise LockdownError(f'Incorrect response returned. Got: {response}')
 
         error = response.get('Error')
         if error is not None:
@@ -562,15 +590,15 @@ class LockdownClient(ABC, LockdownServiceProvider):
                                 'MissingValue': MissingValueError,
                                 'InvalidService': InvalidServiceError,
                                 'InvalidConnection': InvalidConnectionError, }
-            raise exception_errors.get(error, LockdownError)(error)
+            raise exception_errors.get(error, LockdownError)(error, self.identifier)
 
         # iOS < 5: 'Error' is not present, so we need to check the 'Result' instead
         if response.get('Result') == 'Failure':
-            raise LockdownError()
+            raise LockdownError('', self.identifier)
 
         return response
 
-    def _request_pair(self, pair_options: Mapping, timeout: float = None) -> Mapping:
+    def _request_pair(self, pair_options: dict, timeout: Optional[float] = None) -> dict:
         try:
             return self._request('Pair', pair_options)
         except PairingDialogResponsePendingError:
@@ -600,15 +628,15 @@ class LockdownClient(ABC, LockdownServiceProvider):
 
 class UsbmuxLockdownClient(LockdownClient):
     def __init__(self, service: ServiceConnection, host_id: str, identifier: str = None,
-                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
+                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Optional[dict] = None,
                  pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT,
                  usbmux_address: Optional[str] = None):
+        self.usbmux_address = usbmux_address
         super().__init__(service, host_id, identifier, label, system_buid, pair_record, pairing_records_cache_folder,
                          port)
-        self.usbmux_address = usbmux_address
 
     @property
-    def short_info(self) -> Dict:
+    def short_info(self) -> dict:
         short_info = super().short_info
         short_info['ConnectionType'] = self.service.mux_device.connection_type
         return short_info
@@ -634,7 +662,7 @@ class PlistUsbmuxLockdownClient(UsbmuxLockdownClient):
 
 class TcpLockdownClient(LockdownClient):
     def __init__(self, service: ServiceConnection, host_id: str, hostname: str, identifier: str = None,
-                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
+                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Optional[dict] = None,
                  pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT, keep_alive: bool = True):
         """
         Create a LockdownClient instance
@@ -675,7 +703,7 @@ class RemoteLockdownClient(LockdownClient):
         raise NotImplementedError('RemoteXPC lockdown version does not support pairing operations')
 
     def __init__(self, service: ServiceConnection, host_id: str, identifier: str = None,
-                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
+                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Optional[dict] = None,
                  pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT):
         """
         Create a LockdownClient instance
@@ -695,7 +723,7 @@ class RemoteLockdownClient(LockdownClient):
 
 def create_using_usbmux(serial: str = None, identifier: str = None, label: str = DEFAULT_LABEL, autopair: bool = True,
                         connection_type: str = None, pair_timeout: float = None, local_hostname: str = None,
-                        pair_record: Mapping = None, pairing_records_cache_folder: Path = None,
+                        pair_record: Optional[dict] = None, pairing_records_cache_folder: Path = None,
                         port: int = SERVICE_PORT, usbmux_address: Optional[str] = None) -> UsbmuxLockdownClient:
     """
     Create a UsbmuxLockdownClient instance
@@ -715,25 +743,47 @@ def create_using_usbmux(serial: str = None, identifier: str = None, label: str =
     """
     service = ServiceConnection.create_using_usbmux(serial, port, connection_type=connection_type,
                                                     usbmux_address=usbmux_address)
-    cls = UsbmuxLockdownClient
-    with usbmux.create_mux(usbmux_address=usbmux_address) as client:
-        if isinstance(client, PlistMuxConnection):
-            # Only the Plist version of usbmuxd supports this message type
-            system_buid = client.get_buid()
-            cls = PlistUsbmuxLockdownClient
+    try:
+        cls = UsbmuxLockdownClient
+        with usbmux.create_mux(usbmux_address=usbmux_address) as client:
+            if isinstance(client, PlistMuxConnection):
+                # Only the Plist version of usbmuxd supports this message type
+                system_buid = client.get_buid()
+                cls = PlistUsbmuxLockdownClient
 
-    if identifier is None:
-        # attempt get identifier from mux device serial
-        identifier = service.mux_device.serial
+        if identifier is None:
+            # attempt get identifier from mux device serial
+            identifier = service.mux_device.serial
 
-    return cls.create(
-        service, identifier=identifier, label=label, system_buid=system_buid, local_hostname=local_hostname,
-        pair_record=pair_record, pairing_records_cache_folder=pairing_records_cache_folder, pair_timeout=pair_timeout,
-        autopair=autopair, usbmux_address=usbmux_address)
+        return cls.create(
+            service, identifier=identifier, label=label, system_buid=system_buid, local_hostname=local_hostname,
+            pair_record=pair_record, pairing_records_cache_folder=pairing_records_cache_folder,
+            pair_timeout=pair_timeout,
+            autopair=autopair, usbmux_address=usbmux_address)
+    except Exception:
+        service.close()
+        raise
+
+
+def retry_create_using_usbmux(retry_timeout: Optional[float] = None, **kwargs) -> UsbmuxLockdownClient:
+    """
+    Repeatedly retry to create a UsbmuxLockdownClient instance while dismissing different errors that might occur
+    while device is rebooting
+
+    :param retry_timeout: Retry timeout in seconds or None for no timeout
+    :return: UsbmuxLockdownClient instance
+    """
+    start = time.time()
+    while (retry_timeout is None) or (time.time() - start < retry_timeout):
+        try:
+            return create_using_usbmux(**kwargs)
+        except (NoDeviceConnectedError, ConnectionFailedError, BadDevError, OSError, construct.core.StreamError,
+                DeviceNotFoundError):
+            pass
 
 
 def create_using_tcp(hostname: str, identifier: str = None, label: str = DEFAULT_LABEL, autopair: bool = True,
-                     pair_timeout: float = None, local_hostname: str = None, pair_record: Mapping = None,
+                     pair_timeout: float = None, local_hostname: str = None, pair_record: Optional[dict] = None,
                      pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT,
                      keep_alive: bool = False) -> TcpLockdownClient:
     """
@@ -752,16 +802,19 @@ def create_using_tcp(hostname: str, identifier: str = None, label: str = DEFAULT
     :return: TcpLockdownClient instance
     """
     service = ServiceConnection.create_using_tcp(hostname, port, keep_alive=keep_alive)
-    client = TcpLockdownClient.create(
-        service, identifier=identifier, label=label, local_hostname=local_hostname, pair_record=pair_record,
-        pairing_records_cache_folder=pairing_records_cache_folder, pair_timeout=pair_timeout, autopair=autopair,
-        port=port, hostname=hostname, keep_alive=keep_alive)
-    return client
+    try:
+        return TcpLockdownClient.create(
+            service, identifier=identifier, label=label, local_hostname=local_hostname, pair_record=pair_record,
+            pairing_records_cache_folder=pairing_records_cache_folder, pair_timeout=pair_timeout, autopair=autopair,
+            port=port, hostname=hostname, keep_alive=keep_alive)
+    except Exception:
+        service.close()
+        raise
 
 
 def create_using_remote(service: ServiceConnection, identifier: str = None, label: str = DEFAULT_LABEL,
                         autopair: bool = True, pair_timeout: float = None, local_hostname: str = None,
-                        pair_record: Mapping = None, pairing_records_cache_folder: Path = None,
+                        pair_record: Optional[dict] = None, pairing_records_cache_folder: Path = None,
                         port: int = SERVICE_PORT) -> RemoteLockdownClient:
     """
     Create a TcpLockdownClient instance over RSD
@@ -777,8 +830,53 @@ def create_using_remote(service: ServiceConnection, identifier: str = None, labe
     :param port: lockdownd service port
     :return: TcpLockdownClient instance
     """
-    client = RemoteLockdownClient.create(
-        service, identifier=identifier, label=label, local_hostname=local_hostname, pair_record=pair_record,
-        pairing_records_cache_folder=pairing_records_cache_folder, pair_timeout=pair_timeout, autopair=autopair,
-        port=port)
-    return client
+    try:
+        return RemoteLockdownClient.create(
+            service, identifier=identifier, label=label, local_hostname=local_hostname, pair_record=pair_record,
+            pairing_records_cache_folder=pairing_records_cache_folder, pair_timeout=pair_timeout, autopair=autopair,
+            port=port)
+    except Exception:
+        service.close()
+        raise
+
+
+async def get_mobdev2_lockdowns(
+        udid: Optional[str] = None, pair_records: Optional[Path] = None, only_paired: bool = False,
+        timeout: float = DEFAULT_BONJOUR_TIMEOUT) \
+        -> AsyncIterable[tuple[str, TcpLockdownClient]]:
+    records = {}
+    if pair_records is None:
+        pair_records = get_home_folder()
+    for file in pair_records.glob('*.plist'):
+        if file.name.startswith('remote_'):
+            # skip RemotePairing records
+            continue
+        record_udid = file.parts[-1].strip('.plist')
+        if udid is not None and record_udid != udid:
+            continue
+        record = plistlib.loads(file.read_bytes())
+        records[record['WiFiMACAddress']] = record
+
+    iterated_ips = set()
+    for answer in await browse_mobdev2(timeout=timeout):
+        if '@' not in answer.name:
+            continue
+        wifi_mac_address = answer.name.split('@', 1)[0]
+        record = records.get(wifi_mac_address)
+
+        if only_paired and record is None:
+            continue
+
+        for ip in answer.ips:
+            if ip in iterated_ips:
+                # skip ips we already iterated over, possibly from previous queries
+                continue
+            iterated_ips.add(ip)
+            try:
+                lockdown = create_using_tcp(hostname=ip, autopair=False, pair_record=record)
+            except Exception:
+                continue
+            if only_paired and not lockdown.paired:
+                lockdown.close()
+                continue
+            yield ip, lockdown
